package eu.hsrw.tr.prog.vl.arrays;

import java.util.Scanner;

/**
 * Demonstration zur Implementierung endlicher Automaten in Java mit
 * einem Array für die Zustandsübergangsfunktion
 * 
 * Beispiel: Teilbarkeit einer Binärzahl durch 3
 * 
 * @author Thomas Richter
 *
 */
public class EndlicherAutomat {

	public static void main(String[] args) {
		// Zustandsübergangsfunktion:
		//
		//                     |       neuer Zustand nach
		// aktueller Zustand   |   Eingabe 0    |     Eingabe 1
		//                     |                |
		//           0         |       0        |         1
		//           1         |       2        |         0
		//           2         |       1        |         2
		//
		int[][] delta = {{0, 1}, {2, 0}, {1, 2}};
		
		// Binärzahl als String einlesen
		Scanner scan = new Scanner(System.in);
		System.out.print("Eingabe (Binärzahl): ");
		String eingabe = scan.nextLine();
		
		// Startzustand setzen
		int zustand = 0;
		
		// Eingabe zeichenweise durchgehen
		for (int i = 0; i < eingabe.length(); i++) {
			// aktuelles Zeichen der Eingabe
			char ziffer = eingabe.charAt(i);
			
			// neuer Zustand: Eintrag in delta
			// Zeile: aktueller Zustand
			// Spalte: Eingabe
			// ASCII-Code der Ziffer muss noch in 0 oder 1 transformiert werden
			zustand = delta[zustand][ziffer - '0'];
		}
		
		// eingegebene Zahl binär und dezimal ausgeben
		System.out.print(eingabe + " (dezimal " + Integer.parseInt(eingabe, 2) + ")");
	
		// Der Automat akzeptiert die Eingabe, wenn der Endzustand 0 ist
		System.out.println(" teilbar durch 3: " + (zustand == 0));
	}
}